using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Kav
{
    public class LinearDepthEffect : Effect
    {
        EffectParameter modelParam;
        EffectParameter modelViewProjectionParam;
        EffectParameter lightPositionParam;
        EffectParameter farPlaneParam;

        EffectDirtyFlags dirtyFlags = EffectDirtyFlags.All;

        Matrix model;
        Matrix view;
        Matrix projection;

        public Vector3 LightPosition { get; set; }
        public float FarPlane { get; set; }

        public Matrix Model
        {
            get { return model; }
            set
            {
                model = value;
                dirtyFlags |= EffectDirtyFlags.World;
                dirtyFlags |= EffectDirtyFlags.WorldViewProj;
            }
        }

        public Matrix View
        {
            get { return view; }
            set
            {
                view = value;
                dirtyFlags |= EffectDirtyFlags.WorldViewProj;
            }
        }

        public Matrix Projection
        {
            get { return projection; }
            set
            {
                projection = value;
                dirtyFlags |= EffectDirtyFlags.WorldViewProj;
            }
        }

        public LinearDepthEffect(GraphicsDevice graphicsDevice) : base(graphicsDevice, Resources.LinearDepthEffect)
        {
            CacheEffectParameters();
        }

        protected override void OnApply()
        {
            if ((dirtyFlags & EffectDirtyFlags.WorldViewProj) != 0)
            {
                Matrix.Multiply(ref view, ref projection, out Matrix viewProjection);
                Matrix.Multiply(ref model, ref viewProjection, out Matrix worldViewProj);

                modelViewProjectionParam.SetValue(worldViewProj);

                dirtyFlags &= ~EffectDirtyFlags.WorldViewProj;
            }

            if ((dirtyFlags & EffectDirtyFlags.World) != 0)
            {
                modelParam.SetValue(model);
                
                dirtyFlags &= ~EffectDirtyFlags.World;
            }

            lightPositionParam.SetValue(LightPosition);
            farPlaneParam.SetValue(FarPlane);
        }

        private void CacheEffectParameters()
        {
            modelParam               = Parameters["Model"];
            modelViewProjectionParam = Parameters["ModelViewProjection"];

            lightPositionParam       = Parameters["LightPosition"];
            farPlaneParam            = Parameters["FarPlane"];
        }
    }
}
